/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl.guests;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Date;
import javax.mail.MessagingException;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import org.apache.commons.codec.binary.Base64InputStream;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.exception.OXException;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.mail.MailField;
import com.openexchange.mail.dataobjects.MailMessage;
import com.openexchange.mail.dataobjects.SecurityInfo;
import com.openexchange.mail.mime.MimeDefaultSession;
import com.openexchange.mail.mime.converters.MimeMessageConverter;

/**
 * {@link MailMessageFactory} internal simple factory for creating {@link MailMessage} instances from a given OX Guard JSON response
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.10.0
 */
class MailMessageFactory {

    private static final String EMAILS        = "emails";
    private static final String DATA          = "data";
    private static final String CONTENT       = "content";
    private static final String META_DATA     = "metaData";
    private static final String ID            = "id";
    private static final String FOLDER_ID     = "folderId";
    private static final String MESSAGE_FLAGS = "messageFlags";
    private static final String BCC           = "bcc";
    private static final String CC            = "cc";
    private static final String TO            = "to";
    private static final String FROM          = "from";
    private static final String COLOR_LABEL   = "colorLabel";
    private static final String RECEIVED_DATE = "receivedDate";
    private static final String SENT_DATE     = "sentDate";
    private static final String SIZE          = "size";
    private static final String SUBJECT       = "subject";
    private static final String CONTENT_TYPE  = "contentType";

    /**
     * Internal method to create a {@link MimeMessage} from the given data
     *
     * @param input The data
     * @return The {@link MimeMessage} read from the given data
     * @throws MessagingException
     */
    private MimeMessage createMimeMessage(InputStream input) throws MessagingException {
        return input != null ?
            new MimeMessage(MimeDefaultSession.getDefaultSession(), input) :
            new MimeMessage(MimeDefaultSession.getDefaultSession());
    }

    /**
     * Internal method to create a {@link MailMessage} from the given {@link MimeMessage}
     *
     * @param mimeMessage The {@link MimeMessage}
     * @param mailId The unique ID of the mail
     * @return The created {@link MailMessage}
     * @throws OXException
     */
    private MailMessage createMailMessage(MimeMessage mimeMessage, String mailId) throws OXException {
        final MailField[] FIELDS = {MailField.BODY, MailField.HEADERS};
        MailMessage mailMessage =
            MimeMessageConverter.convertMessage(mimeMessage,
                mailId,
                "GUARD-GUEST",
                com.openexchange.mail.MailPath.SEPERATOR,
                FIELDS);
        mailMessage.setMailId(mailId);
        SecurityInfo securityInfo = new SecurityInfo(true, false);
        mailMessage.setSecurityInfo(securityInfo);
        return mailMessage;
    }

    /**
     * Internal method to apply the meta data to given {@link MailMessage} object
     *
     * @param mailMessage The message to apply the meta data for
     * @param metaData The meta data to apply
     * @throws OXException
     * @throws JSONException
     * @throws AddressException
     */
    private void applyMetaData(MailMessage mailMessage, JSONObject metaData) throws OXException, JSONException, AddressException {
        if(!metaData.isNull(CONTENT_TYPE)) {
            mailMessage.setContentType(metaData.getString(CONTENT_TYPE));
        }

        if(!metaData.isNull(SUBJECT)) {
            mailMessage.setSubject(metaData.getString(SUBJECT));
        }

        if(!metaData.isNull(SIZE)) {
            mailMessage.setSize(metaData.getLong(SIZE));
        }

        if(!metaData.isNull(SENT_DATE)) {
            mailMessage.setSentDate(new Date(metaData.getLong(SENT_DATE)));
        }

        if(!metaData.isNull(RECEIVED_DATE)) {
            mailMessage.setReceivedDate(new Date(metaData.getLong(RECEIVED_DATE)));
        }

        if(!metaData.isNull(COLOR_LABEL)) {
            mailMessage.setColorLabel(metaData.getInt(COLOR_LABEL));
        }

        if(!metaData.isNull(FROM)) {
            JSONArray jsonArray = metaData.getJSONArray(FROM);
            for(int i=0; i< jsonArray.length(); i++) {
               mailMessage.addFrom(new InternetAddress(jsonArray.getString(i)));
            }
        }

        if(!metaData.isNull(TO)) {
            JSONArray jsonArray = metaData.getJSONArray(TO);
            for(int i=0; i< jsonArray.length(); i++) {
               mailMessage.addTo(new InternetAddress(jsonArray.getString(i)));
            }
        }

        if(!metaData.isNull(CC)) {
            JSONArray jsonArray = metaData.getJSONArray(CC);
            for(int i=0; i< jsonArray.length(); i++) {
               mailMessage.addCc(new InternetAddress(jsonArray.getString(i)));
            }
        }

        if(!metaData.isNull(BCC)) {
            JSONArray jsonArray = metaData.getJSONArray(BCC);
            for(int i=0; i< jsonArray.length(); i++) {
               mailMessage.addBcc(new InternetAddress(jsonArray.getString(i)));
            }
        }

        if(!metaData.isNull(MESSAGE_FLAGS)) {
            mailMessage.setFlags(metaData.getInt(MESSAGE_FLAGS));
        }

        if(!metaData.isNull(FOLDER_ID)) {
            mailMessage.setFolder(metaData.getString(FOLDER_ID));
        }

    }

    /**
     * Internal method to create a {@link MailMessage} from the given JSON object
     *
     * @param json The JSON to create the mail message form
     * @return The created mail message
     * @throws OXException
     */
    private MailMessage createaMailMessageFrom(JSONObject json) throws OXException {
        try {
            final String mailId = json.getString(ID);
            final String content = !json.isNull(CONTENT) ? json.getString(CONTENT) : null;
            final MimeMessage mimeMessage = createMimeMessage(
                content != null ?
                new Base64InputStream(new ByteArrayInputStream(content.getBytes(StandardCharsets.UTF_8))) :
                null);

            MailMessage mailMessage = createMailMessage(mimeMessage, mailId);
            applyMetaData(mailMessage, json.getJSONObject(META_DATA));

            return mailMessage;
        } catch (JSONException | MessagingException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }
    }

    /**
     * Parses the given JSONObject to a set of {@link MailMessage} instances
     *
     * @param json The JSONObject to parse
     * @return A set of parsed {@link MailMessages}
     * @throws OXException
     */
    MailMessage[] createFrom(JSONObject json) throws OXException {
        try {
            JSONObject data = json.getJSONObject(DATA);
            JSONArray emails = data.getJSONArray(EMAILS);
            MailMessage[] ret = new MailMessage[emails.length()];
            for (int i = 0; i < emails.length(); i++) {
                JSONObject email = emails.getJSONObject(i);
                ret[i] = createaMailMessageFrom(email);
            }
            return ret;
        } catch (JSONException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }
    }
}
